#
#  db.awk  Ver 1.0
#
#  <C> Copyright 1995 Symbiotic Solutions
#  Authors:  Emanuel Noik
#
#  This code may look like awk but it's actually gawk!

#-------------------------------------------------------------------------------

# Initialize database 'package'
#
function dbInit() {
   IGNORECASE = 1;

   dbComparVal[EQ] = 1;
   dbComparVal[NE] = 2;
   dbComparVal[LT] = 3;
   dbComparVal[LE] = 4;
   dbComparVal[GT] = 5;
   dbComparVal[GE] = 6;

   dbErr = 0;
   dbLastField = 0;
   dbOffset=1;
   dbMaxHits=999999;
   dbStart = systime();
}

#-------------------------------------------------------------------------------

# Process command line arguments; create:
#
#   qa[field] = 0 or 1		and
#   qc[field] = ComparF		comparison function
#   qv[field,i]=value		i'th value of field
#
function dbArgs(    i,n,v,t,len,a,b) {
   len = split(args,b,"+");
   for(i = 1; i <= len; i++ )
   {
      split(b[i],a,"=");

      # additional field values
      if( int(a[1]) == 0 && a[1] != "0" )
      {
         t = substr(a[1],1,1);
         n = int(substr(a[1],2,length(a[1])-1));
         if( t == "a" )
         {
            if( int(a[2]) == 0 && a[2] != "0" )
            {
               if( a[2] ~ "AND" ) qa[n] = 1;
               else qa[n] = 0;
            }
            else
               qa[n] = int(a[2]);
         }
         else if( t == "c" )
         {
            if( int(a[2]) == 0 && a[2] != "0" )
               qc[n] = dbComparVal[a[2]];
            else
               qc[n] = int(a[2]);
         }
         else if( t == "Y" )	# process date members
         {
            qv[n,1] = a[2];
            qn[n]++;
         }
         else if( t == "M" )
         {
            qv[n,2] = a[2];
            qn[n]++;
         }
         else if( t == "D" )
         {
            qv[n,3] = a[2];
            qn[n]++;
         }
         else if( a[1] == "maxHits" )
         {
            dbMaxHits = int(a[2]);
            if( dbMaxHits > 50 )	# prevent malicious use
               dbMaxHits = 50;
         }
         else if( a[1] == "offset" )
         {
            dbOffset = int(a[2]);
         }
         else
         {
            dbErr = 1;
            #exit
         }
      }
      # regular field values
      else
      {
         n = int(a[1]);
         v = a[2];
         qn[n]++;
         # must catch label instead of value in options here...
         if( qt[n]=="L" || qt[n]=="M" )
         {
            #if( int(v) == 0 && a[2] != "0" )
            #{
            #   v = mk[a[1],a[2]];
            #}
         }
         qv[n,qn[n]] = v;
      }
   }
}

#-------------------------------------------------------------------------------

# Create a field:
#
#   qt[field]=type		"N" "D" "L" "M" "S"
#   qn[i]=no of values
#
function dbField(i,type,path) {
   dbLastField = dbLastField > i ? dbLastField : i;
   qt[i]=type;
   qn[i]=0;
   if( qt[i]=="L" || qt[i]=="M" )
   {
      qa[i]=0;
      if( path != "" )
         dbMask(i,sprintf("%s/lib/%s",rootpath,path));
   }
}

#-------------------------------------------------------------------------------

# Read mask table; create:
#
#   mk[field,label] = key
#   ml[field,key]   = label
#
function dbMask(i,path,    rc) {
   for( ; 1 ; )
   {
      rc = getline < path
      if( rc <= 0 ) break;
      mk[i,$2] = $1;
      ml[i,$1] = $2;
   }
}

#-------------------------------------------------------------------------------

# Filter field i
#
function dbFilterI(i,    x,j,n,a,b,len) {
   if( qn[i] == 0 ) return 0;
   if( qt[i] == "S" )
   {
      if( $i ~ qv[i,1] ) return 1;
      else return -1
   }
   else if( qt[i] == "L" )
   {
      for( k = 1; k <= qn[i]; k++ )
         if( qv[i,k] == "" )
            return 1;
         else if( $i ~ qv[i,k] )
            return 1;
      return -1;
   }
   else if( qt[i] == "N" || qt[i] == "D" )
   {
      return Compar(i);
   }
   else if( qt[i] == "M" )
   {
      n = 0;
      len = split($i,b,",");
      for( j = 1; j <= len; j++ )
      {
         for( k = 1; k <= qn[i]; k++ )
            if( qv[i,k] == "" )
               return 1;
            else if( b[j] ~ qv[i,k] )
               n++;
      }
      if( qa[i] == 1 )
      {
         if( n == qn[i] ) return 1;
      }
      else 
      {
         if( n > 0 ) return 1;
      }
      return -1
   }
   return 0;
}

#-------------------------------------------------------------------------------

# Compare dates and numbers.
# Note, a DATE value is a string of the form YYMMDD
# which can be compared alphabetically rather than numerically.
#
function Compar(i     ,v) {
   if( qt[i] != "D" && qt[i] != "N" ) return 0;
   if( qc[i] == 0 ) return 0;
   v = qt[i] == "N" ? qv[i,1] : sprintf("%s%s%s",qv[i,1],qv[i,2],qv[i,3]);
   if( qc[i] == 1 ) return $i == v ? 1 : -1;
   if( qc[i] == 2 ) return $i != v ? 1 : -1;
   if( qc[i] == 3 ) return $i <  v ? 1 : -1;
   if( qc[i] == 4 ) return $i <= v ? 1 : -1;
   if( qc[i] == 5 ) return $i >  v ? 1 : -1;
   if( qc[i] == 6 ) return $i >= v ? 1 : -1;
   return 0;
}

#-------------------------------------------------------------------------------

# Dump a form for fetching the next 'n' hits.
#
function dbMoreHits(script) {
  print "<form action=\"" script "\" method=\"POST\">";
  print " <input type=\"hidden\" name=\"maxHits\" value=" dbMaxHits ">";
  print " <input type=\"hidden\" name=\"offset\" value=" NR ">";
  for ( i = 1; i <= dbLastField; i++ )
     for( j = 1; j <= qn[i]; j++ )
         print " <input type=\"hidden\" name=\"" i "\" value=\"" qv[i,j] "\">";
  print " <input type=\"submit\" value=\"Next " dbMaxHits " matches\">";
  print "</form>";
}

#-------------------------------------------------------------------------------

function dbFooter() {
   dbEnd = systime();
   printf( "Retrieved %d %s in %d %s.\n", n,
      n == 1 ? "match" : "matches", dbEnd - dbStart,
      dbEnd - dbStart == 1 ? "second" : "seconds" );
}

#-------------------------------------------------------------------------------
